# FSExternalMethod.py
# modified from FSExternalMethod from FileSystemSite

# differences:
# - cleanups to get working under CMF (different imports)
# - index_html = None  (critical -- don't inherit the index but use __call__)
# - can now customize -> folder

from os import path

import Globals
import Acquisition

from AccessControl import ClassSecurityInfo, getSecurityManager
from Products.ExternalMethod.ExternalMethod import ExternalMethod

from Products.CMFCore.utils import _dtmldir
from Permissions import ViewManagementScreens, View
from Products.CMFCore.DirectoryView import registerFileExtension, registerMetaType, expandpath
from Products.CMFCore.FSObject import FSObject

import os
from Globals import package_home

import Permissions

## still need hookpoint for reloading metadata

class FSExternalMethod(ExternalMethod, FSObject):

    meta_type = 'FileSystem External Method'

    manage_options=(
        (
            {'label':'Customize', 'action':'manage_main'},
            {'label':'Test', 'action':''}
            ),
        )

    security = ClassSecurityInfo()
    security.declareObjectProtected( Permissions.View )

    security.declareProtected( Permissions.ViewManagementScreens
                             , 'manage_main')
    manage_main = Globals.DTMLFile( 'custext', 
            os.path.join( package_home(globals()), 'dtml' ))

    allowed_params = ('title', 'module', 'function')

    def __init__(self, id, filepath, fullname=None, properties=None):
        FSObject.__init__(self, id, filepath, fullname, properties)

    def _createZODBClone(self):
        """
            Create a ZODB (editable) equivalent of this object.
        """
        # XXX:  do this soon
        return ExternalMethod(self.getId(), self.title, self.module(), self.function())

    def _readFile( self, reparse ):
        fp = expandpath( self._filepath )
        file = open( fp, 'r' ) 
        parameters = {}
        
        try:
            for line in file.readlines():
                line = line.strip()
                if not line:
                    continue                    
                if line[0] == '#':
                    continue
                key, value = line.split(':')
                parameters[key.strip()] = value.strip()
                
        finally:
            file.close()

        for k in parameters.keys():
            if k not in self.allowed_params:
                del parameters[k]
        
        self.manage_edit(**parameters)

    index_html = None

    if Globals.DevelopmentMode:
        # Provide an opportunity to update the properties.
        def __of__(self, parent):
            try:
                self = Acquisition.ImplicitAcquisitionWrapper(self, parent)
                self._updateFromFS()
                return self
            except:
                from zLOG import LOG, ERROR
                import sys
                LOG('FS External Method',
                    ERROR,
                    'Error during __of__',
                    error=sys.exc_info())
                raise

Globals.InitializeClass(FSExternalMethod)

registerFileExtension('ext', FSExternalMethod)
registerMetaType('External Method', FSExternalMethod)

